// mozileUtilities.js

/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *
 * ***** END LICENSE BLOCK ***** */

/*******

Mozile Utilities Javascript
Mozile version 0.6.18

Created by James A. Overton
Created on 2004-03-24

Last revised by James A. Overton
Last revised on 2005-02-04

HISTORY:
	Some code in this file was first found in mozilehtmltb.js, while the 
majority was cut out of mozileTB.js and separated into different files.


DESCRIPTION:
	General functions which belong nowhere else.

*********/

/*
Mozile Debug Levels
0 = Really important, alert the user right away, add to debug array
1 = Important, always show this in the status, add to debug array
2 = Not important, add to debug array
3 = Unimportant, add to debug array

*/
	
	
/* Max moved this block to mozileConfiguration.js

// Listen for onbeforeunload events, and pass them to the mozileOnBeforeUnload handler.
document.addEventListener("beforeunload", mozileOnBeforeUnload, false);

//Note: The onbeforeunload event seems to fire when the browser starts up. The warning is only
//produced if Mozile has been injected.
function mozileOnBeforeUnload(event) {
	var f=["mozileUtilities.js","mozileOnBeforeUnload"];
	mozileDebug(f,2,"Page unload handler.");
	
	netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect');
    var mozilePrefs = Components.classes["@mozilla.org/preferences-service;1"].
        getService(Components.interfaces.nsIPrefBranch);

	try {
		// if the promptOnUnloadBool preference is "false", do nothing
		if(!mozilePrefs.getBoolPref("mozile.promptOnUnloadBool")) {
			mozileDebug(f,2,"Page unload ignored.");
			return false;
		} else {
			// Make mTarg the document that is being closed
			var mTarg;
			if ( event.target.nodeType == Node.DOCUMENT_NODE ) {
				mTarg = event.target;
			}
			else {
				mTarg = event.target.ownerDocument;
			}

			if ( "MOZILE_CURRENT_CONFIG" in mTarg ) {
				if(mTarg.MOZILE_CURRENT_CONFIG.saved !== "true") {
					//If page hasn't been saved, warn user.
					mozileDebug(f,1,"Unload event:"+ event + event.target +" "+ event.target.nodeType +" "+ mTarg.documentURI);
					mozileDebug(f,1,"Unloading Document");
					event.stopPropagation();
					event.returnValue = false;
					event.preventDefault();
					return false;
				}
			}
		}  	
  }
	catch (e) {
		mozileDebug(f,2,"Exception: Nothing to unload? "+e);
	}
	return true;
}
*/

var mozileDebugArray = new Array();

function mozileDebug(f,level,msg) {
	var mozileDebugLevel = mozilePrefs.getIntPref("mozile.debugLevelInt");
	
	// act only if the level of the message is higher than the mozileDebugLevel
	
	if( (level > 1 && mozileDebugLevel < 1) || (level > 2 && mozileDebugLevel < 2) ) {
		// Ignore these debug events
	}
	else {
		// Log to the debugging array	
		var datestamp = 
		mozileDD('FullYear')	+"-"+
		mozileDD('Month')			+"-"+
		mozileDD('Date')			+" "+
		mozileDD('Hours')			+":"+
		mozileDD('Minutes')		+":"+
		mozileDD('Seconds');

		var entry = new Array(datestamp,f,level,msg);
		mozileDebugArray.push(entry);

		// Critical behaviours
		if(level <= 1) {
  		var message = "Mozile: ["+ f +"] "+ msg; 
  		if(level == 0) {
  			alert(message);
  		}
  		if(level == 1) {
  			window.status = message.substr(0,250);
  		}
		}
	}
}

	// Returns date strings as two digits (with leading "0") if they are 1 digit long
var mozileDates = new Object();

function mozileDD(command) {
		// first see if we've already recorded it...
	if(command in mozileDates) {
		return mozileDates[command];
	}
	
	var d = new Date();
	var commandString = "d.get"+command+"()";
	var str = eval(commandString)+"";
	if(str.length==1) {
		str = "0"+str;
	}
	if(command=="FullYear" || command=="Month" || command=="Date") {
		mozileDates[command]=str;
	}

	return str;
}

function mozileDebugAlert() {
	alert(mozileDebugArray);
}


function mozileGetContext(sel) {
	var f = ["mozileUtilities.js","mozileGetContext"];

	try {
		if ( sel.anchorNode != null ) {
		  var aN = sel.anchorNode;
			if( aN.nodeType == Node.ELEMENT_NODE ) {
//				mozileDebug(f,3,"Selection anchorNode: "+aN.nodeName);
				return aN;
 			}
 			if ( aN.parentNode != null ) {
				var aNpN = aN.parentNode;
   			if( aNpN.nodeType == Node.ELEMENT_NODE ) {
// 					mozileDebug(f,3,"Selection parentNode: "+aNpN.nodeName);
   				return aNpN;
   			}
			}
		}
		else {
			mozileDebug(f,1,"No Node!");
		}
	}
	catch(e) {
		mozileDebug(f,1,"Exception: "+e);
		return null;
	}
	return null;
}



function mozileRemoveChildren(node) {	
	var f = ["mozileUtilities.js","mozileRemoveChildren"];
	mozileDebug(f,3,"Removing Children of: "+ node + node.id );
	
	if(node.hasChildNodes()!=true) {
		return node;
	}
	var children = node.childNodes;	
	mozileDebug(f,3,"Number of Children: "+children.length);
	for(var i=0; i<children.length; i++) {
		node.removeChild(children[i]);
	}
	return node;
}



function mozileAlert(name) {
	alert("Mozile Alert: "+name);
}	


function printXML(XML) {
	var objXMLSerializer = new XMLSerializer;
	output = objXMLSerializer.serializeToString(XML);
		// fix stupid uppercase tags!
	output = output.replace( /<(\/?)([A-Z]+)/g,
		function (output,p1,p2,offset,s) {
			return '<'+p1+p2.toLowerCase() ;
		} ) ;
	return output;
}

function dumpText(text) {
	var ta = document.getElementById("here");
	ta = mozileRemoveChildren(ta);
	var tn = document.createTextNode(text);
	ta.appendChild(tn);
}

function dumpXML(XML) {
	dumpText(printXML(XML));
}



// The following functions have to do with entity insertion
// str refers to a character code. See this site: http://www.w3.org/TR/REC-html40/sgml/entities.html
function mozileInsertEntity(str) {
	var textNode = mozileTarget.document.createTextNode(String.fromCharCode(str));
	
		// split the current text node
	mozileSelection.anchorNode.splitText(mozileSelection.anchorOffset);
	
		// insert the text element 			
	var parentNode = mozileSelection.anchorNode.parentNode;
	var nextNode = mozileSelection.anchorNode.nextSibling;
	parentNode.insertBefore(textNode, nextNode);
	mozileSelection.moveInsertionPoint(mozileTarget.Selection.MOVE_FORWARD_ONE);

	mozileDo("Special Character");
	mozileUpdateSourceView();
}



var mozileLastEA = "";

// The following function concern the source editing features.

function mozileUpdateSourceView() {
	var f = ["mozileUtilities.js","mozileUpdateSourceView"];
	mozileDebug(f,3,"Source is being updated...");

	if(!mozileSourceBox || mozileSourceBox==""){
		mozileSourceBox = document.getElementById("mozileSourceBox");
	}	
		// if the source box doesn't exist or is not visible, don't bother updating.
	if(!mozileSourceBox || mozileSourceBox.style.visibility=="collapse") return;
	
	try {
		if(mozileTarget) {
			var editableArea="";
			try {
				var cssr = mozileSelection.getEditableRange();
				editableArea = cssr.top;
				mozileLastEA=editableArea;
				mozileDebug(f,2,"Editable Area: "+editableArea);
			} catch(e) {
				mozileDebug(f,2,"No Editable Area: "+e);
				editableArea=mozileLastEA;
			}
				// Decide whether to use HTML or XML mode
			if(mozileTarget.document.contentType=="text/html") {
				mozileSourceBox.value = mozileGetInnerHTML(editableArea);
			}
			else {
				mozileSourceBox.value = mozileGetInnerXML(editableArea);
			}
		
		}
		else {
			mozileSourceBox.value = "No editable selection.";
		}
	} catch(e) {
		mozileDebug(f,2,"Error getting innerHTML: "+e);
	}
}


function mozileUpdateMarkupView(node) {
	var f = ["mozileUtilities.js","mozileUpdateMarkupView"];
	mozileDebug(f,3,"Markup is being updated...");
	
	try {
		var editableArea="";
		try {
			var cssr = mozileSelection.getEditableRange();
			editableArea = cssr.top;
			mozileLastEA=editableArea;
			mozileDebug(f,2,"Editable Area: "+editableArea);
		} catch(e) {
			mozileDebug(f,2,"No Editable Area: "+e);
			editableArea=mozileLastEA;
		}
			// Decide whether to use HTML or XML mode
		if(mozileTarget.document.contentType=="text/html") {
			mozileSetInnerHTML(editableArea, node.value);
		}
		else {
			mozileSetInnerXML(editableArea, node.value);
		}
		//mozileDo("Source Editing");
	} catch(e) {
		mozileDebug(f,2,"Error setting innerXML or innerHTML: "+e);
	}
}

	// toggles the visibility of the source view box
function mozileShowSourceView() {
	var f = ["mozileUtilities.js","mozileShowSourceView"];
	mozileDebug(f,3,"Toggling source view...");
	//alert("Show Source!");
	var button = document.getElementById("mozile-edit-source");
	
		// First see we have the splitter and box already
	mozileDebug(f,3,"Source tools: "+ mozileSourceSplitter + mozileSourceBox);
	if( !mozileSourceSplitter || !mozileSourceBox ) {
			// If tey aren't defined, then try to grab them
		mozileDebug(f,3,"Detecting source tools");
		var mozileSourceSplitter = document.getElementById("mozileSourceSplitter");
		var mozileSourceBox = document.getElementById("mozileSourceBox");
		mozileDebug(f,3,"Source tools: "+ mozileSourceSplitter + mozileSourceBox);
			// If they're still not there, create them.
		if( !mozileSourceSplitter || !mozileSourceBox ) {
			mozileDebug(f,3,"Creating source tools");
				// Grab the container
			var mozileSourceVbox = document.getElementById("appcontent");
				// Create the splitter
			mozileSourceSplitter = document.createElement("splitter");
			mozileSourceSplitter.id = "mozileSourceSplitter";
				// Create the textbox
			mozileSourceBox = document.createElement("textbox");
			mozileSourceBox.id = "mozileSourceBox";
			mozileSourceBox.setAttribute("multiline","true");
			mozileSourceBox.setAttribute("height","200px");
			mozileSourceBox.setAttribute("persist","height");
			mozileSourceBox.setAttribute("oninput","mozileUpdateMarkupView(this)");
				// Append the new elements
			mozileSourceVbox.appendChild(mozileSourceSplitter);
			mozileSourceVbox.appendChild(mozileSourceBox);
			mozileSourceSplitter.style.visibility = "collapse";
			mozileSourceBox.style.visibility = "collapse";
		}
	}
		// If I use the "collapse" XUL attribute then the textbox won't trigger oninput events, so instead we use the "visibility" CSS property
	if(mozileSourceBox.style.visibility == "collapse") {
		mozileSourceSplitter.style.visibility = "visible";
		mozileSourceBox.style.visibility = "visible";
		button.setAttribute("active","true");
	}
	else {
		mozileSourceSplitter.style.visibility = "collapse";
		mozileSourceBox.style.visibility = "collapse";
		button.setAttribute("active","false");
	}
	
	mozileUpdateSourceView();
}


// Functions to get and set the content of an element

function mozileSetInnerXML(node,str) {
	var f = ["mozileUtilities.js","mozileSetInnerXML"];
	mozileDebug(f,3,"Setting inner XML...");
	try {
		var xdoc = mozileStrToDOM(str);
		mozileDebug(f,2,"XDOC: "+ xdoc +" => "+ printXML(xdoc));
		mozileReplaceNodeWithDF(node, xdoc);
	} catch(e) {
		mozileDebug(f,1,"Set Inner XML Error: "+e);
	}
	return str;
	
}

function mozileGetInnerXML(node) {
	var f = ["mozileUtilities.js","mozileGetInnerXML"];
	mozileDebug(f,3,"Getting inner XML...");

	//create a new XMLSerializer
	try {
		var objXMLSerializer = new XMLSerializer;
	} catch(e) {
		mozileDebug(f,0,"Your browser does not seem to be equipped with the required 'XML Extras'. These are standard in the mozilla.org browser releases, but may not be included in some custom releases. Mozile cannot save documents without XMLSerializer. There was an error when trying to create a new XMLSerializer: "+e);
		return false;
	}
	
	var dataToSaveRange = document.createRange();
	dataToSaveRange.selectNodeContents(node);
	var dataToSave = dataToSaveRange.cloneContents();

	var contentToSave = objXMLSerializer.serializeToString(dataToSave);
	
	return contentToSave;
}



function mozileSetInnerHTML(node,str){
	var r = node.ownerDocument.createRange();
	r.selectNodeContents(node);
	r.deleteContents();
	var df = r.createContextualFragment(str);
	node.appendChild(df);
	return str;
}


function mozileGetInnerHTML(node){
/*
	var str = "";
	for (var i=0; i<node.childNodes.length; i++)
		str += node.ownerDocument.getOuterHTML(node.childNodes.item(i));
	return str;
*/
	return node.innerHTML;
}






// Call a few functions that should only be called once everything else has been done.
// Killed in favour of the new Firefox update mechanism
// mozileAutoUpdate();